#include "pixmap.h"

#include <stdio.h>
#include <stdlib.h>

pixmapT *alloc_pixmap(int rows, int cols)
{
    pixmapT *pixmap;
    int i;

    /* allocate pixmap */
    if ((pixmap = malloc(sizeof(*pixmap))) == NULL) {
	return NULL;
    }
    pixmap->rows = rows;
    pixmap->cols = cols;
    if ((pixmap->pixels = malloc(rows * sizeof(*pixmap->pixels))) == NULL) {
	return NULL;
    }
    for (i = 0; i < rows; i++) {
	if ((pixmap->pixels[i] = calloc(cols, sizeof(**pixmap->pixels))) == NULL) {
	    return NULL;
	}
    }
    return pixmap;
}

void pixmap_replace(
		       pixmapT * dst,
		       pixmapT * src,
		       int xoff, int yoff
)
{
    int i, j;
    for (i = 0; i < src->rows; i++) {
	for (j = 0; j < src->cols; j++) {
	    int rs = red(src->pixels[i][j]);
	    int gs = green(src->pixels[i][j]);
	    int bs = blue(src->pixels[i][j]);
	    int as = alpha(src->pixels[i][j]);
	    int rd = red(dst->pixels[i + yoff][j + xoff]);
	    int gd = green(dst->pixels[i + yoff][j + xoff]);
	    int bd = blue(dst->pixels[i + yoff][j + xoff]);
	    int ad = alpha(dst->pixels[i + yoff][j + xoff]);

	    dst->pixels[i + yoff][j + xoff] =
		pixel(
			 ad,
			 (rs * as + rd * (255 - as)) / 255,
			 (gs * as + gd * (255 - as)) / 255,
			 (bs * as + bd * (255 - as)) / 255);
	}
    }
}

void write_pixmap(const char *fn, pixmapT * pixmap)
{
    FILE *fp;
    int x, y;

    if (strcmp(fn, "-") == 0) {
	fp = stdout;
    } else if ((fp = fopen(fn, "wb")) == NULL) {
	perror(fn);
	exit(1);
    }
    fprintf(fp, "P6\n%d %d\n255\n", pixmap->cols, pixmap->rows);
    for (y = 0; y < pixmap->rows; y++) {
	for (x = 0; x < pixmap->cols; x++) {
	    putc(red(pixmap->pixels[y][x]), fp);
	    putc(green(pixmap->pixels[y][x]), fp);
	    putc(blue(pixmap->pixels[y][x]), fp);
	}
    }
    if (fp != stdout)
	fclose(fp);
}
