/* ttf.h */

#ifndef _TTF_H
#define _TTF_H 1

#include <X11/Xmd.h>
#include <X11/Xlib.h>

#include "pixmap.h"

/************************************************************************/
/* These definitions are from <X11/Xmd.h> */
/* */
/* BYTE		8 bit value */
/* INT8		8 bit signed integer */
/* INT16	16 bit signed integer */
/* INT32	32 bit signed integer */
/* CARD8	8 bit unsigned integer */
/* CARD16	16 bit unsigned integer */
/* CARD32	32 bit unsigned integer */
/* BOOL		True or False */
/* STRING8	LISTofCARD8 */
/* STRING16	LISTofCHAR2B */
/* POINT	[X,Y:INT16] */
/* RECTANGLE	[X,Y:INT16  width,height:CARD16] */
/************************************************************************/

typedef INT16 FWord;		/* signed distances in the em-square */
typedef CARD16 uFWord;		/* unsigned distances in the em-square */
typedef INT32 Fixed;		/* signed 16.16 fixed point */
typedef INT16 F2Dot14;		/* signed 2.14 fixed point */
typedef CARD8 DateTime[8];	/* International date (8-byte field) */

/************************************************************************/

#define TTF_INT32(p) ((INT32)(((INT32)(((CARD8 *)p)[0]) << 24) \
			      | ((INT32)(((CARD8 *)p)[1]) << 16) \
			      | ((INT32)(((CARD8 *)p)[2]) << 8) \
			      | ((INT32)(((CARD8 *)p)[3]))))

#define TTF_CARD32(p) ((CARD32)(((CARD32)(((CARD8 *)p)[0]) << 24) \
				| ((CARD32)(((CARD8 *)p)[1]) << 16) \
				| ((CARD32)(((CARD8 *)p)[2]) << 8) \
				| ((CARD32)(((CARD8 *)p)[3]))))

#define TTF_INT16(p) ((INT16)(((INT16)(((CARD8 *)p)[0]) << 8) \
			      | ((INT16)(((CARD8 *)p)[1]))))

#define TTF_CARD16(p) ((CARD32)(((CARD16)(((CARD8 *)p)[0]) << 8) \
				| ((CARD16)(((CARD8 *)p)[1]))))

#define TTF_CARD8(p) (((CARD8 *)(p))[0])

#define TTF_FWord(p) ((FWord)TTF_INT16(p))
#define TTF_uFWord(p) ((uFWord)TTF_CARD16(p))
#define TTF_Fixed(p) ((Fixed)TTF_INT32(p))
#define TTF_F2Dot14(p) ((F2Dot14)TTF_INT16(p))

/************************************************************************/

typedef struct tag_ttfOffsetTable {
    Fixed sfntVersion;		/* 0x00010000 for version 1.0. */
    CARD16 numTables;		/* Number of tables. */
    CARD16 searchRange;		/* (Maximum power of 2 <= numTables) x 16. */
    CARD16 entrySelector;	/* Log2(maximum power of 2 <= numTables) */
    CARD16 rangeShift;		/* NumTables x 16-searchRange. */
} ttfOffsetTable;

/************************************************************************/

typedef struct tag_ttfTableDirectory {
    CARD32 tag;			/* 4-byte identifier */
    CARD32 checkSum;		/* CheckSum for this table */
    CARD32 offset;		/* Offset from the beginning of the file */
    CARD32 length;		/* Length of this table */
} ttfTableDirectory;

/************************************************************************/

typedef struct tag_ttfFontHeader {
    Fixed version;		/* Table version number			*/
    Fixed fontRevision;		/* Set by font manufacturer.		*/
    CARD32 checkSumAdjustment;	/* To compute: set it to 0, sum the 	*/
    /* entire font as CARD32, then store 	*/
    /* 0xB1B0AFBA - sum.			*/
    CARD32 magicNumber;		/* Set to 0x5F0F3CF5.			*/
    CARD16 flags;		/* Bit 0 - baseline for font at y=0;	*/
    /* Bit 1 - left sidebearing at x=0;	*/
    /* Bit 2 - instructions may be 		*/
    /*         dependant on point16 size;	*/
    /* Bit 3 - force ppem to integer values */
    /*         for all internal scaler math */
    /*         may use fractional ppem 	*/
    /*         sizes if this bit is clear;	*/
    /* Bit 4 - instructions may alter 	*/
    /*         advance width (i.e., the 	*/
    /*         advance widths might not 	*/
    /*         scale linearly); 		*/
    /* Note: All other bits must be zero.	*/
    CARD16 unitsPerEm;		/* Valid range is from 16 to 16384	*/
    DateTime created;		/* International date (8-byte field).	*/
    DateTime modified;		/* International date (8-byte field).	*/
    FWord xMin, yMin, xMax, yMax;	/* For all glyph bounding boxes.	*/
    CARD16 macStyle;		/* Bit 0 - bold (if set to 1)		*/
    /* Bit 1 - italic (if set to 1)		*/
    /* Bits 2-15 reserved (set to 0).	*/
    CARD16 lowestRecPPEM;	/* Smallest readable size in pixels.	*/
    short fontDirectionHint;	/* 0 Fully mixed directional glyphs; 	*/
    /* 1 Only strongly lseft to right;	*/
    /* 2 Like 1 but also contains neutrals	*/
    /* -1 Only strongly right to left;	*/
    /* -2 Like -1 but also contains neutrals*/
    short indexToLocFormat;	/* 0 for short offsets, 1 for CARD32.	*/
    short glyphDataFormat;	/* 0 for current format.		*/
}

ttfFontHeader;

/************************************************************************/

typedef struct tag_ttfGlyphData {
    CARD16 numberOfContours;	/* If the number of contours is greater	*/
    /* than or equal to zero, this is a	*/
    /* single glyph; if negative, this is a	*/
    /* composite glyph.			*/
    FWord xMin, yMin;		/* Minimum x and y for coordinate data.	*/
    FWord xMax, yMax;		/* Maximum x for coordinate data.	*/
} ttfGlyphData;

/* For decoding multi-component glyph bit flags */
#define ARG_1_AND_2_ARE_WORDS     0x0001
#define ARGS_ARE_XY_VALUES        0x0002
#define ROUND_XY_TO_GRID          0x0004
#define WE_HAVE_A_SCALE           0x0008
/*      RESERVED                  0x0010 */
#define MORE_COMPONENTS           0x0020
#define WE_HAVE_AN_X_AND_Y_SCALE  0x0040
#define WE_HAVE_A_TWO_BY_TWO      0x0080
#define WE_HAVE_INSTRUCTIONS      0x0100
#define USE_MY_METRICS            0x0200

/************************************************************************/

typedef struct tag_ttfMaxProfile {
    Fixed version;		/* Version, 0x00010000 for version 1.0 */
    CARD16 numGlyphs;		/* The number of glyphs in the font */
    CARD16 maxPoints;		/* Maximum points in a non-composite glyph */
    CARD16 maxContours;		/* Maximum contours in a non-composite glyph */
    CARD16 maxCompositePoints;	/* Maximum points in a composite glyph. */
    CARD16 maxCompositeContours;/* Maximum contours in a composite glyph. */
    CARD16 maxZones;		/* 1 if instructions do not use the twilight */
    /* zone	(Z0), or 2 if instructions do use */
    /* Z0; should be set to 2 in most cases. */
    CARD16 maxTwilightPoints;	/* Maximum points used in Z0. */
    CARD16 maxStorage;		/* Number of Storage Area locations. */
    CARD16 maxFunctionDefs;	/* Number of FDEFs. */
    CARD16 maxInstructionDefs;	/* Number of IDEFs. */
    CARD16 maxStackElements;	/* Maximum stack depth */
    /* This includes Font and CVT Programs, as */
    /* well as the instructions for each glyph.. */
    CARD16 maxSizeOfInstructions;	/* Maximum byte count for glyph instructions */
    CARD16 maxComponentElements;/* Maximum number of composites referenced */
    /* at "top level" for any composite glyph. */
    CARD16 maxComponentDepth;	/* Maximum levels of recursion; 1 for */
    /* simple components. */
} ttfMaxProfile;

/************************************************************************/

typedef struct tag_ttfFormat4Seg {
    CARD16 endCount;
    CARD16 startCount;
    CARD16 idDelta;
    CARD16 idRangeOffset;
    CARD16 *glyphTable;
} ttfFormat4Seg;

typedef struct tag_ttfCharToIndexMap {
    CARD16 platformID;
    CARD16 specificID;
    CARD32 offset;
    CARD16 format;
    CARD16 length;
    CARD16 version;

    /* Format 4 */
    CARD16 segCount;
    CARD16 searchRange;
    CARD16 entrySelector;
    CARD16 rangeShift;
    ttfFormat4Seg *format4Segs;
} ttfCharToIndexMap;

typedef struct tag_ttfHMetric {
    int advanceWidth;
    int lsb;
}

ttfHMetric;

/************************************************************************/

typedef struct tag_ttfHandle {
    int fileLength;
    const BYTE *fileData;

    const ttfOffsetTable *offsetTable;

    int numTables;
    const ttfTableDirectory *tableDirectory;

    const ttfFontHeader *fontHeader;

    const char *baseGlyphData;

    const char *location;
    const char *hhea;
    const char *hmtx;

    const ttfMaxProfile *maxProfile;

    int nGlyphs;
    const ttfGlyphData **glyphData;

    const char *baseCharToIndexMap;

    int nCharMaps;
    ttfCharToIndexMap **charToIndexMap;

    int unitsPerEm;
    int xMin, yMin, xMax, yMax;

    int ascender;
    int descender;
    int linegap;
    int advanceWidthMax;
    int minLeftSideBearing;
    int minRightSideBearing;
    int xMaxExtent;
    int metricDataFormat;
    int numberOfHMetrics;

    /* horizontal metrics */
    ttfHMetric *hmtxTable;
}

ttfHandle;

/************************************************************************/

#define TTFPOINT_ON 1

typedef struct tag_ttfPoint {
    int x, y;
    int flags;
}

ttfPoint;

/************************************************************************/

typedef struct tag_ttfGlyph {
    long xMin, yMin, xMax, yMax;
    int metricIndex;

    int nContours;

    unsigned endPoints[64];

    int nPoints;

    ttfPoint points[1024];
}

ttfGlyph;

/************************************************************************/

typedef struct tag_ttfRenderList {
    int nColumns;
    short columns[64];
    BOOL on[64];
}

ttfRenderList;

/************************************************************************/

#define TTFTAG(a, b, c, d) ((CARD32)a << 24 | (CARD32)b << 16 \
			    | (CARD32)c << 8 | (CARD32)d)

#define TTFMAGIC 0x5F0F3CF5
#define TTFCHECKSUMADJ 0xB1B0AFBA

/************************************************************************/

#if HARDCODED_SCALING
#define FRACTION 64
#define FRACTMASK 63

#define SCALE (FRACTION/16)

#define TERMVAL (FRACTION/2)

#endif

#define DEPTH 10

/************************************************************************/

ttfHandle *ttfOpen(const char *fn);
int ttfCode2Glyph(ttfHandle * handle, int code, int platformID, int specificID);
ttfGlyph *ttfReadGlyph(ttfHandle * handle, int iGlyph);
XPoint *ttfLinearize(XPoint * lines, int nPoints, ttfPoint * points);
void ttfRender(ttfRenderList * renderList, int nLines, XPoint * lines);
void ttfRasterize(
		     pixmapT * bitmap,
		     int xoff,
		     ttfRenderList * renderList);

int fraction;

/************************************************************************/

#endif				/* TTF_H */
